//=============================================================================
// ScoreRecovery.
//=============================================================================
/*
Score Recovery v4 install notes:

1. Close unreal/server
2. Copy ScoreRecovery4.u to unreal\system\ dir
3. Open unreal.ini, scroll down to the [Engine.GameEngine] section and add:
ServerPackages=ScoreRecovery4
ServerActors=ScoreRecovery4.ScoreRecovery
4. Start server

Score Recovery was originally made by Smartball, v4 was retouched by SA|BarendB.
*/


class ScoreRecovery extends ReplicationInfo;

struct TPlayerScoreInfo {
	var SRClInfo ClientInfo;
	var int iScore, iCommSent, iUID;
	var bool bIsFinished, bReceived, bSpawned;
};

var TPlayerScoreInfo PlayerInfo[100];
var string CurrentTeamNames[2];
var bool bTeam;


Function PostBeginPlay()
{
	bTeam = (TeamGame(Level.Game) != None);
	If(bTeam)
	{
		CurrentTeamNames[0]=TeamGame(Level.Game).Teams[0].TeamName;
		CurrentTeamNames[1]=TeamGame(Level.Game).Teams[1].TeamName;
	}
	SetTimer(3, True);
	Super.PostBeginPlay();
}

function Timer()
{
   	local PlayerReplicationInfo PRI;
   	local Pawn P;
   	local int iScore;
   
   	for(P=Level.PawnList; P!=None; P=P.NextPawn)
	{
		if ( !P.IsA('Bots') && !P.IsA('Spectator') && !P.IsA('ScriptedPawn') && !P.IsA('FlockPawn') && !P.IsA('FlockMasterPawn') )
 		{
  			PRI = P.PlayerReplicationInfo;
			if (!PlayerInfo[PRI.PlayerID].bIsFinished) 
			{
				if (!PlayerInfo[PRI.PlayerID].bSpawned)
				{
					PlayerInfo[PRI.PlayerID].ClientInfo = P.Spawn(Class'SRClInfo', P);
					PlayerInfo[PRI.PlayerID].bSpawned = True;
				}
				
				if (!PlayerInfo[PRI.PlayerID].bReceived)
				{
					if ( (PlayerInfo[PRI.PlayerID].iCommSent == 0) || (PlayerInfo[PRI.PlayerID].iCommSent != PlayerInfo[PRI.PlayerID].ClientInfo.g_iCommRet) )
					{
						PlayerInfo[PRI.PlayerID].iCommSent = (rand(0x7FFE) + 1) * (rand(0x7FFE) + 1);
						PlayerInfo[PRI.PlayerID].ClientInfo.GetUID(PlayerInfo[PRI.PlayerID].iCommSent, False);
					}
					else
					{					
						if (!CheckUID(PlayerInfo[PRI.PlayerID].ClientInfo.g_iUID))
						{
							PlayerInfo[PRI.PlayerID].iCommSent = (rand(0x7FFE) + 1) * (rand(0x7FFE) + 1);
							PlayerInfo[PRI.PlayerID].ClientInfo.GetUID(PlayerInfo[PRI.PlayerID].iCommSent, True);
						}
						else
						{
							PlayerInfo[PRI.PlayerID].iUID = PlayerInfo[PRI.PlayerID].ClientInfo.g_iUID;
							PlayerInfo[PRI.PlayerID].bReceived = True;
							Log("ScoreRecovery: Player " $ PRI.PlayerName $ "'s Unique ID = " $ PlayerInfo[PRI.PlayerID].iUID);
							PlayerInfo[PRI.PlayerID].ClientInfo.Destroy();
						}
					}
				}
				
				if ( (PlayerInfo[PRI.PlayerID].bReceived) && ((bTeam && IsValidTeam(PRI.TeamName)) || (!bTeam)) )
				{
 	 				iScore = GetScore(PlayerInfo[PRI.PlayerID].iUID, PRI.PlayerID);
 					if (iScore != 0)
 					{
 						PRI.Score = iScore;
						BroadCastMessage(PRI.PlayerName $ "'s score has been recovered!");						}
					PlayerInfo[PRI.PlayerID].bIsFinished = True;
				}
			}		
 			PlayerInfo[PRI.PlayerID].iScore = PRI.Score;
		}
  	}
}


function int GetScore (int iUID, int iCurrentID)
{
	local int i, iScore;
	
	iScore = 0;
  	for (i = 0; i < iCurrentID; i++)
  	{
		if (PlayerInfo[i].bIsFinished)
		{
			if (iUID == PlayerInfo[i].iUID)
				iScore = PlayerInfo[i].iScore;
		}
	}
	return iScore;
}


function bool CheckUID (int iUID)
{
   	local PlayerReplicationInfo PRI;
   	local Pawn P;
   	
   	for(P=Level.PawnList; P!=None; P=P.NextPawn)
	{
		if ( !P.IsA('Bots') && !P.IsA('Spectator') && !P.IsA('ScriptedPawn') && !P.IsA('FlockPawn') && !P.IsA('FlockMasterPawn') )
 		{
  			PRI = P.PlayerReplicationInfo;
			if (PlayerInfo[PRI.PlayerID].iUID == iUID) 
				return False;
		}
	}		
	return True;
}


function bool IsValidTeam (string TeamCheck)
{
	return (TeamCheck ~= CurrentTeamNames[0]) || (TeamCheck ~= CurrentTeamNames[1]);
}

defaultproperties
{
}
